@@ -3,17 +3,24 @@ import collections
 import logging
 import os
 import sys
+import warnings
 
 import sqlalchemy as sa
 import sqlparse
 from sqlalchemy.dialects import postgresql
+from sqlalchemy.dialects.postgresql import array
 from sqlalchemy.orm import sessionmaker
+from sqlalchemy.sql import Values
 
 from .constants import (
     BUILTIN_SCHEMAS,
+    FOREIGN_KEY_INDEX,
+    FOREIGN_KEY_VIEW,
     LOGICAL_SLOT_PREFIX,
     LOGICAL_SLOT_SUFFIX,
     PLUGIN,
+    PRIMARY_KEY_INDEX,
+    PRIMARY_KEY_VIEW,
     SCHEMA,
     TG_OP,
     TRIGGER_FUNC,
@@ -23,13 +30,25 @@ from .exc import ForeignKeyError, ParseLogicalSlotError, TableNotFoundError
 from .settings import PG_SSLMODE, PG_SSLROOTCERT, QUERY_CHUNK_SIZE
 from .trigger import CREATE_TRIGGER_TEMPLATE
 from .utils import get_postgres_url
+from .view import CreateIndex, CreateView, DropIndex, DropView
+
+try:
+    import citext  # noqa
+except ImportError:
+    pass
+
+try:
+    import geoalchemy2  # noqa
+except ImportError:
+    pass
+
 
 logger = logging.getLogger(__name__)
 
 
 class Base(object):
 
-    def __init__(self, database, *args, **kwargs):
+    def __init__(self, database, verbose=False, *args, **kwargs):
         """Initialize the base class constructor.
 
         Args:
@@ -37,9 +56,10 @@ class Base(object):
         """
         self.__engine = pg_engine(database, **kwargs)
         self.__schemas = None
-        # models is a dict of f'{schema}.{table}''
+        # models is a dict of f'{schema}.{table}'
         self.models = {}
         self.__metadata = {}
+        self.verbose = verbose
 
     def connect(self):
         """Connect to database."""
@@ -51,15 +71,18 @@ class Base(object):
             raise
 
     def pg_settings(self, column):
-        statement = sa.select([sa.column('setting')])
-        statement = statement.select_from(sa.text('pg_settings'))
-        statement = statement.where(sa.column('name') == column)
-        if self.verbose:
-            compiled_query(statement, 'pg_settings')
-        try:
-            return self.query_one(statement)[0]
-        except Exception:
-            pass
+        row = self.fetchone(
+            sa.select([
+                sa.column('setting')
+            ]).select_from(
+                sa.text('pg_settings')
+            ).where(
+                sa.column('name') == column
+            ),
+            label='pg_settings',
+        )
+        if row:
+            return row[0]
         return None
 
     def has_permission(self, username, permission):
@@ -76,14 +99,18 @@ class Base(object):
         if permission not in ('usecreatedb', 'usesuper', 'userepl'):
             raise RuntimeError(f'Invalid user permission {permission}')
 
-        statement = sa.select([sa.column(permission)])
-        statement = statement.select_from(sa.text('pg_user'))
-        statement = statement.where(sa.column('usename') == username)
-        statement = statement.where(sa.column(permission) == True) # noqa
-        if self.verbose:
-            compiled_query(statement, 'has_permission')
         try:
-            return self.query_one(statement)[0]
+            return self.fetchone(
+                sa.select([sa.column(permission)]).select_from(
+                    sa.text('pg_user')
+                ).where(
+                    sa.and_(*[
+                        sa.column('usename') == username,
+                        sa.column(permission) == True, # noqa
+                    ])
+                ),
+                label='has_permission',
+            )[0]
         except Exception as e:
             logger.exception(f'{e}')
         return False
@@ -113,8 +140,17 @@ class Base(object):
                 )
             model = metadata.tables[name]
             model.append_column(sa.Column('xmin', sa.BigInteger))
+            model.append_column(sa.Column('oid', sa.dialects.postgresql.OID))
             model = model.alias()
+            setattr(
+                model,
+                'primary_keys',
+                sorted([
+                    primary_key.key for primary_key in model.primary_key
+                ]),
+            )
             self.models[f'{model.original}'] = model
+
         return self.models[name]
 
     @property
@@ -165,12 +201,6 @@ class Base(object):
             f'Invalid definition {table} for schema: {schema}'
         )
 
-    def _get_table(self, schema, table):
-        """Get fully qualified table name."""
-        if not table.startswith(f'{schema}.'):
-            return f'{schema}.{table}'
-        return table
-
     def truncate_table(self, table, schema=SCHEMA):
         """Truncate a table.
 
@@ -183,7 +213,8 @@ class Base(object):
             schema (str): The database schema
 
         """
-        table = self._get_table(schema, table)
+        if not table.startswith(f'{schema}.'):
+            table = f'{schema}.{table}'
         schema, table = self._get_schema(schema, table)
         logger.debug(f'Truncating table: {schema}.{table}')
         query = f'TRUNCATE TABLE "{schema}"."{table}" CASCADE'
@@ -218,18 +249,18 @@ class Base(object):
 
         SELECT * FROM PG_REPLICATION_SLOTS
         """
-        statement = sa.select(['*'])
-        statement = statement.select_from(
-            sa.text('PG_REPLICATION_SLOTS')
+        return self.fetchall(
+            sa.select(['*']).select_from(
+                sa.text('PG_REPLICATION_SLOTS')
+            ).where(
+                sa.and_(*[
+                    sa.column('slot_name') == slot_name,
+                    sa.column('slot_type') == slot_type,
+                    sa.column('plugin') == plugin,
+                ])
+            ),
+            label='replication_slots',
         )
-        statement = statement.where(
-            sa.and_(*[
-                sa.column('slot_name') == slot_name,
-                sa.column('slot_type') == slot_type,
-                sa.column('plugin') == plugin,
-            ])
-        )
-        return self.query(statement)
 
     def create_replication_slot(self, slot_name):
         """
@@ -242,13 +273,15 @@ class Base(object):
         SELECT * FROM PG_REPLICATION_SLOTS
         """
         logger.debug(f'Creating replication slot: {slot_name}')
-        statement = sa.select(['*']).select_from(
-            sa.func.PG_CREATE_LOGICAL_REPLICATION_SLOT(
-                slot_name,
-                PLUGIN,
-            )
+        return self.fetchone(
+            sa.select(['*']).select_from(
+                sa.func.PG_CREATE_LOGICAL_REPLICATION_SLOT(
+                    slot_name,
+                    PLUGIN,
+                )
+            ),
+            label='create_replication_slot',
         )
-        return self.query_one(statement)
 
     def drop_replication_slot(self, slot_name):
         """
@@ -259,11 +292,13 @@ class Base(object):
         """
         logger.debug(f'Dropping replication slot: {slot_name}')
         if self.replication_slots(slot_name):
-            statement = sa.select(['*']).select_from(
-                sa.func.PG_DROP_REPLICATION_SLOT(slot_name),
-            )
             try:
-                return self.query_one(statement)
+                return self.fetchone(
+                    sa.select(['*']).select_from(
+                        sa.func.PG_DROP_REPLICATION_SLOT(slot_name),
+                    ),
+                    label='drop_replication_slot',
+                )
             except Exception as e:
                 logger.exception(f'{e}')
                 raise
@@ -286,8 +321,9 @@ class Base(object):
         SELECT * FROM PG_LOGICAL_SLOT_GET_CHANGES('testdb', NULL, NULL)
         """
         filters = []
-        columns = [sa.column('xid'), sa.column('data')]
-        statement = sa.select(columns)
+        statement = sa.select(
+            [sa.column('xid'), sa.column('data')]
+        )
         statement = statement.select_from(
             sa.func.PG_LOGICAL_SLOT_GET_CHANGES(
                 slot_name,
@@ -311,9 +347,7 @@ class Base(object):
             )
         if filters:
             statement = statement.where(sa.and_(*filters))
-        if self.verbose:
-            compiled_query(statement, 'logical_slot_get_changes')
-        return self.query(statement)
+        return self.fetchall(statement, label='logical_slot_get_changes')
 
     def logical_slot_peek_changes(
         self,
@@ -329,8 +363,9 @@ class Base(object):
         SELECT * FROM PG_LOGICAL_SLOT_PEEK_CHANGES('testdb', NULL, 1)
         """
         filters = []
-        columns = [sa.column('xid'), sa.column('data')]
-        statement = sa.select(columns).select_from(
+        statement = sa.select(
+            [sa.column('xid'), sa.column('data')]
+        ).select_from(
             sa.func.PG_LOGICAL_SLOT_PEEK_CHANGES(
                 slot_name,
                 upto_lsn,
@@ -353,16 +388,261 @@ class Base(object):
             )
         if filters:
             statement = statement.where(sa.and_(*filters))
-        if self.verbose:
-            compiled_query(statement, 'logical_slot_peek_changes')
-        return self.query(statement)
+        return self.fetchall(statement, label='logical_slot_peek_changes')
+
+    # Views...
+    def _primary_key_view_statement(self, schema, tables, views):
+        """
+        Table name and primary keys association where the table name
+        is the index.
+        This is only called once on bootstrap.
+        It is used within the trigger function to determine what payload
+        values to send to pg_notify.
+
+        Since views cannot be modified, we query the existing view for exiting
+        rows and union this to the next query.
+
+        So if 'specie' was the only row before, and the next query returns
+        'unit' and 'structure', we want to end up with the result below.
+
+        table_name | primary_keys
+        ------------+--------------
+        specie     | {id}
+        unit       | {id}
+        structure  | {id}
+        """
+
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore', category=sa.exc.SAWarning)
+            pg_class = self.model('pg_class', 'pg_catalog')
+            pg_index = self.model('pg_index', 'pg_catalog')
+            pg_attribute = self.model('pg_attribute', 'pg_catalog')
+            pg_namespace = self.model('pg_namespace', 'pg_catalog')
+
+        alias = pg_class.alias('x')
+        statement = sa.select([
+            sa.cast(
+                sa.cast(
+                    pg_index.c.indrelid,
+                    sa.dialects.postgresql.REGCLASS,
+                ), sa.Text,
+            ).label(
+                'table_name'
+            ),
+            sa.func.ARRAY_AGG(pg_attribute.c.attname).label(
+                'primary_keys'
+            ),
+        ]).join(
+            pg_attribute,
+            pg_attribute.c.attrelid == pg_index.c.indrelid,
+        ).join(
+            pg_class,
+            pg_class.c.oid == pg_index.c.indexrelid,
+        ).join(
+            alias,
+            alias.c.oid == pg_index.c.indrelid,
+        ).join(
+            pg_namespace,
+            pg_namespace.c.oid == pg_class.c.relnamespace,
+        ).where(*[
+            pg_namespace.c.nspname.notin_(
+                ['pg_catalog', 'pg_toast']
+            ),
+            pg_index.c.indisprimary,
+            sa.cast(
+                sa.cast(
+                    pg_index.c.indrelid,
+                    sa.dialects.postgresql.REGCLASS,
+                ), sa.Text,
+            ).in_(tables),
+            pg_attribute.c.attnum == sa.any_(pg_index.c.indkey),
+        ]).group_by(
+            pg_index.c.indrelid
+        )
+
+        if PRIMARY_KEY_VIEW in views:
+            values = self.fetchall(sa.select([
+                sa.column('table_name'),
+                sa.column('primary_keys'),
+            ]).select_from(
+                sa.text(PRIMARY_KEY_VIEW)
+            ))
+            self.__engine.execute(DropView(schema, PRIMARY_KEY_VIEW))
+            if values:
+                statement = statement.union(
+                    sa.select(
+                        Values(
+                            sa.column('table_name'),
+                            sa.column('primary_keys'),
+                        ).data(
+                            [(value[0], array(value[1])) for value in values]
+                        ).alias(
+                            't'
+                        )
+                    )
+                )
+
+        return statement
+
+    def _foreign_key_view_statement(self, tables):
+        """
+        Table name and foreign keys association where the table name
+        is the index.
+        This is also only called once on bootstrap.
+        It is used within the trigger function to determine what payload
+        values to send to pg_notify.
+
+        table_name | foreign_keys
+        ------------+--------------
+        specie     | {id}
+        unit       | {id}
+        structure  | {id}
+        """
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore', category=sa.exc.SAWarning)
+            table_constraints = self.model(
+                'table_constraints',
+                'information_schema',
+            )
+            key_column_usage = self.model(
+                'key_column_usage',
+                'information_schema',
+            )
+            constraint_column_usage = self.model(
+                'constraint_column_usage',
+                'information_schema',
+            )
+
+        return sa.select([
+            table_constraints.c.table_name,
+            sa.func.ARRAY_AGG(
+                sa.cast(
+                    key_column_usage.c.column_name,
+                    sa.TEXT,
+                )
+            ).label('foreign_keys'),
+        ]).join(
+            key_column_usage,
+            sa.and_(
+                key_column_usage.c.constraint_name == table_constraints.c.constraint_name,
+                key_column_usage.c.table_schema == table_constraints.c.table_schema,
+             )
+        ).join(
+            constraint_column_usage,
+            sa.and_(
+                constraint_column_usage.c.constraint_name == table_constraints.c.constraint_name,
+                constraint_column_usage.c.table_schema == table_constraints.c.table_schema,
+             )
+        ).where(*[
+             table_constraints.c.table_name.in_(tables),
+             table_constraints.c.constraint_type == 'FOREIGN KEY',
+        ]).group_by(
+            table_constraints.c.table_name
+        )
+
+    def create_views(self, schema, tables, user_defined_fkey_tables):
+
+        views = sa.inspect(self.engine).get_view_names(schema)
+
+        logger.debug(f'Creating view: {schema}.{PRIMARY_KEY_VIEW}')
+        self.__engine.execute(
+            CreateView(
+                schema,
+                PRIMARY_KEY_VIEW,
+                self._primary_key_view_statement(schema, tables, views),
+            )
+        )
+        self.__engine.execute(DropIndex(PRIMARY_KEY_INDEX))
+        self.__engine.execute(
+            CreateIndex(
+                PRIMARY_KEY_INDEX,
+                schema,
+                PRIMARY_KEY_VIEW,
+                ['table_name'],
+            )
+        )
+        logger.debug(f'Created view: {schema}.{PRIMARY_KEY_VIEW}')
+
+        logger.debug(f'Creating view: {schema}.{FOREIGN_KEY_VIEW}')
+
+        # if view exists, query it first
+        rows = {}
+        if FOREIGN_KEY_VIEW in views:
+            _rows = self.fetchall(
+                sa.select([
+                    sa.column('table_name').label('table_name'),
+                    sa.func.ARRAY_AGG(
+                        sa.column('fkeys')
+                    ).label(
+                        'foreign_keys'
+                    ),
+                ]).select_from(
+                    sa.text(FOREIGN_KEY_VIEW),
+                    sa.func.unnest(
+                        sa.column('foreign_keys')
+                    ).alias(
+                        'fkeys'
+                    )
+                ).group_by(
+                    sa.column('table_name')
+                )
+            )
+            rows = _rows_to_dict(_rows)
+
+        _rows = self.fetchall(
+            self._foreign_key_view_statement(tables)
+        )
+        if _rows:
+            _rows = _rows_to_dict(_rows)
+            rows = _merge_dict(rows, _rows)
+
+        if user_defined_fkey_tables:
+            rows = _merge_dict(rows, user_defined_fkey_tables)
+
+        # if view exists, drop it first, we have all the existing data
+        if FOREIGN_KEY_VIEW in views:
+            self.__engine.execute(DropView(schema, FOREIGN_KEY_VIEW))
+
+        statement = sa.select(
+            Values(
+                sa.column('table_name'),
+                sa.column('foreign_keys'),
+            ).data([
+                (key, array(value)) for key, value in rows.items()
+            ]).alias(
+                'v'
+            )
+        )
+        self.__engine.execute(
+            CreateView(
+                schema,
+                FOREIGN_KEY_VIEW,
+                statement,
+            )
+        )
+        self.__engine.execute(DropIndex(FOREIGN_KEY_INDEX))
+        self.__engine.execute(
+            CreateIndex(
+                FOREIGN_KEY_INDEX,
+                schema,
+                FOREIGN_KEY_VIEW,
+                ['table_name'],
+            )
+        )
+        logger.debug(f'Created view: {schema}.{FOREIGN_KEY_VIEW}')
+
+    def drop_views(self, schema):
+        for view in [PRIMARY_KEY_VIEW, FOREIGN_KEY_VIEW]:
+            logger.debug(f'Dropping view: {schema}.{view}')
+            self.__engine.execute(DropView(schema, view))
+            logger.debug(f'Dropped view: {schema}.{view}')
 
     # Triggers...
-    def create_triggers(self, database, schema, tables=None):
+    def create_triggers(self, schema, tables=None):
         """Create a database triggers."""
         self.execute(CREATE_TRIGGER_TEMPLATE)
-        queries = []
         views = sa.inspect(self.engine).get_view_names(schema)
+        queries = []
         for table in self.tables(schema):
             schema, table = self._get_schema(schema, table)
             if (tables and table not in tables) or (table in views):
@@ -372,20 +652,19 @@ class Base(object):
                 ('notify', 'ROW', ['INSERT', 'UPDATE', 'DELETE']),
                 ('truncate', 'STATEMENT', ['TRUNCATE']),
             ]:
-
-                if self.exist_trigger(f'{table}_{name}', f'{schema}.{table}'):
-                    continue
-
                 queries.append(
-                    f'CREATE TRIGGER {table}_{name} '
-                    f'AFTER {" OR ".join(tg_op)} ON "{schema}"."{table}" '
-                    f'FOR EACH {for_each} EXECUTE PROCEDURE '
-                    f'{TRIGGER_FUNC}()',
+                    sa.DDL(
+                        f'CREATE TRIGGER {table}_{name} '
+                        f'AFTER {" OR ".join(tg_op)} ON "{schema}"."{table}" '
+                        f'FOR EACH {for_each} EXECUTE PROCEDURE '
+                        f'{TRIGGER_FUNC}()',
+                    )
                 )
+
         for query in queries:
             self.execute(query)
 
-    def drop_triggers(self, database, schema, tables=None):
+    def drop_triggers(self, schema, tables=None):
         """Drop all pgsync defined triggers in database."""
         for table in self.tables(schema):
             schema, table = self._get_schema(schema, table)
@@ -399,7 +678,7 @@ class Base(object):
                 )
                 self.execute(query)
 
-    def disable_triggers(self, database, schema):
+    def disable_triggers(self, schema):
         """Disable all pgsync defined triggers in database."""
         for table in self.tables(schema):
             schema, table = self._get_schema(schema, table)
@@ -411,7 +690,7 @@ class Base(object):
                 )
                 self.execute(query)
 
-    def enable_triggers(self, database, schema):
+    def enable_triggers(self, schema):
         """Enable all pgsync defined triggers in database."""
         for table in self.tables(schema):
             schema, table = self._get_schema(schema, table)
@@ -423,68 +702,6 @@ class Base(object):
                 )
                 self.execute(query)
 
-    def exist_trigger(self, tgname, tgrelid):
-        """Check if the given trigger exists on table.
-
-        Args:
-            tgname (str): The trigger name
-            tgrelid (str): The table name
-
-        Returns:
-            True if exists, False otherwise.
-        """
-        statement = sa.text(
-            f"SELECT tgname FROM pg_trigger "
-            f"WHERE NOT tgisinternal "
-            f"AND tgrelid = '{tgrelid}'::REGCLASS "
-            f"AND tgname = '{tgname}' "
-        )
-        return self.query_one(statement) is not None
-
-    def execute(self, query, values=None, options=None):
-        """Execute a query command."""
-        conn = self.__engine.connect()
-        try:
-            if options:
-                conn = conn.execution_options(**options)
-            conn.execute(query, values)
-            conn.close()
-        except Exception as e:
-            logger.exception(f'Exception {e}')
-            raise
-
-    def update(self, query):
-        """Update query command."""
-        conn = self.__engine.connect()
-        try:
-            conn.execute(query)
-            conn.close()
-        except Exception as e:
-            logger.exception(f'Exception {e}')
-            raise
-
-    def fetchall(self, query):
-        """Fetch all rows from a query."""
-        conn = self.__engine.connect()
-        try:
-            rows = conn.execute(query).fetchall()
-            conn.close()
-        except Exception as e:
-            logger.exception(f'Exception {e}')
-            raise
-        return rows
-
-    def fetchone(self, query):
-        """Fetch one row query."""
-        conn = self.__engine.connect()
-        try:
-            row = conn.execute(query).fetchone()
-            conn.close()
-        except Exception as e:
-            logger.exception(f'Exception {e}')
-            raise
-        return row
-
     @property
     def txid_current(self):
         """
@@ -492,10 +709,10 @@ class Base(object):
 
         SELECT txid_current()
         """
-        statement = sa.select(['*']).select_from(
-            sa.func.TXID_CURRENT()
-        )
-        return self.fetchone(statement)[0]
+        return self.fetchone(
+            sa.select(['*']).select_from(sa.func.TXID_CURRENT()),
+            label='txid_current',
+        )[0]
 
     def parse_value(self, type_, value):
         """
@@ -534,7 +751,7 @@ class Base(object):
             'varchar',
         ):
             value = value.lstrip("'").rstrip("'")
-        if type_.lower() in ('boolean',):
+        if type_.lower() == 'boolean':
             try:
                 value = bool(value)
             except ValueError:
@@ -616,52 +833,53 @@ class Base(object):
 
         return payload
 
-    def get_columns(self, model, column_names=None):
-        if column_names:
-            return [
-                getattr(
-                    model.__table__.c, column_name
-                ) for column_name in column_names
-            ]
-        return [column for column in model.__table__.columns]
-
-    def get_column_names(self, model):
-        return model.__table__.columns.keys()
-
-    def get_column_labels(self, columns, column_labels):
-        for i, column in enumerate(columns):
-            if column.name in column_labels:
-                column_label = column_labels[column.name]
-                columns[i] = column.label(column_label)
-        return columns
-
     # Querying...
 
-    def query_one(self, query):
+    def execute(self, statement, values=None, options=None):
+        """Execute a query statement."""
         conn = self.__engine.connect()
         try:
-            row = conn.execute(query).fetchone()
+            if options:
+                conn = conn.execution_options(**options)
+            conn.execute(statement, values)
+            conn.close()
+        except Exception as e:
+            logger.exception(f'Exception {e}')
+            raise
+
+    def fetchone(self, statement, label=None, literal_binds=False):
+        """Fetch one row query."""
+        if self.verbose:
+            compiled_query(statement, label=label, literal_binds=literal_binds)
+
+        conn = self.__engine.connect()
+        try:
+            row = conn.execute(statement).fetchone()
             conn.close()
         except Exception as e:
             logger.exception(f'Exception {e}')
             raise
         return row
 
-    def query(self, query):
+    def fetchall(self, statement, label=None, literal_binds=False):
+        """Fetch all rows from a query statement."""
+        if self.verbose:
+            compiled_query(statement, label=label, literal_binds=literal_binds)
+
         conn = self.__engine.connect()
         try:
-            rows = conn.execute(query).fetchall()
+            rows = conn.execute(statement).fetchall()
             conn.close()
         except Exception as e:
             logger.exception(f'Exception {e}')
             raise
         return rows
 
-    def query_yield(self, query, chunk_size=None):
+    def fetchmany(self, statement, chunk_size=None):
         chunk_size = chunk_size or QUERY_CHUNK_SIZE
         with self.__engine.connect() as conn:
             result = conn.execution_options(stream_results=True).execute(
-                query.select()
+                statement.select()
             )
             while True:
                 chunk = result.fetchmany(chunk_size)
@@ -670,12 +888,13 @@ class Base(object):
                 for keys, row, *primary_keys in chunk:
                     yield keys, row, primary_keys
 
-    def query_count(self, query):
+    def count(self, statement):
         with self.__engine.connect() as conn:
-            query = query.original.with_only_columns(
-                [sa.func.count()]
-            ).order_by(None)
-            return conn.execute(query).scalar()
+            return conn.execute(
+                statement.original.with_only_columns([
+                    sa.func.count()
+                ]).order_by(None)
+            ).scalar()
 
 
 # helper methods
@@ -699,10 +918,6 @@ def subtransactions(session):
     return ControlledExecution(session)
 
 
-def get_primary_keys(model):
-    return sorted([column.key for column in model.primary_key])
-
-
 def _get_foreign_keys(model_a, model_b):
 
     foreign_keys = collections.defaultdict(list)
@@ -833,9 +1048,8 @@ def pg_execute(engine, query, values=None, options=None):
 
 def create_schema(engine, schema):
     """Create database schema."""
-    if schema == SCHEMA:
-        return
-    engine.execute(sa.schema.CreateSchema(schema))
+    if schema != SCHEMA:
+        engine.execute(sa.schema.CreateSchema(schema))
 
 
 def create_database(database, echo=False):
@@ -870,37 +1084,13 @@ def drop_extension(database, extension, echo=False):
     logger.debug(f'Dropped extension: {extension}')
 
 
-def create_materialized_view(database, view, query, echo=False):
-    """Create a materialized database view."""
-    logger.debug(f'Creating materialized view: {view} with {query}')
-    engine = pg_engine(database=database, echo=echo)
-    pg_execute(engine, f'CREATE MATERIALIZED VIEW {view} AS {query}')
-    logger.debug(f'Created materialized view: {view}')
-
-
-def refresh_materialized_view(database, view, echo=False):
-    """Refresh a materialized database view."""
-    logger.debug(f'Refreshing materialized view: {view}')
-    engine = pg_engine(database=database, echo=echo)
-    pg_execute(engine, f'REFRESH MATERIALIZED VIEW {view}')
-    logger.debug(f'Refreshed materialized view: {view}')
-
-
-def drop_materialized_view(database, view, echo=False):
-    """Drop a materialized database view."""
-    logger.debug(f'Dropping materialized view: {view}')
-    engine = pg_engine(database=database, echo=echo)
-    pg_execute(engine, f'DROP MATERIALIZED VIEW IF EXISTS {view}')
-    logger.debug(f'Dropped materialized view: {view}')
-
-
-def compiled_query(query, label=None):
+def compiled_query(query, label=None, literal_binds=False):
     """Compile an SQLAlchemy query with an optional label."""
     query = str(
         query.compile(
             dialect=postgresql.dialect(),
             compile_kwargs={
-                'literal_binds': True
+                'literal_binds': literal_binds
             },
         )
     )
@@ -911,3 +1101,30 @@ def compiled_query(query, label=None):
     else:
         logging.debug(f'{query}')
         sys.stdout.write(f'{query}')
+
+
+def _rows_to_dict(rows):
+    """
+    Converts list of tuples to dict of sets
+        e.g
+            [('author', ['city_id']), ('book', ['publisher_id'])]
+        becomes {
+            {'author': {'city_id'}, 'book': {'publisher_id'}}
+        }
+    """
+    values = {}
+    for key, value in rows:
+        if key in values:
+            values[key] |= set(value)
+            continue
+        values[key] = set(value)
+    return values
+
+
+def _merge_dict(rows1, rows2):
+    for key, value in rows2.items():
+        if key in rows1:
+            rows1[key] |= set(value)
+            continue
+        rows1[key] = set(value)
+    return rows1
