@@ -41,8 +41,8 @@ RELATIONSHIP_ATTRIBUTES = [
 
 # Relationship foreign keys
 RELATIONSHIP_FOREIGN_KEYS = [
-     'child',
-     'parent',
+    'child',
+    'parent',
 ]
 
 
@@ -107,6 +107,38 @@ ELASTICSEARCH_TYPES = [
     'time',
 ]
 
+# https://www.elastic.co/guide/en/elasticsearch/reference/current/mapping-params.html
+ELASTICSEARCH_MAPPING_PARAMETERS = [
+    'analyzer',
+    'boost',
+    'coerce',
+    'copy_to',
+    'doc_values',
+    'dynamic',
+    'eager_global_ordinals',
+    'enabled',
+    'fielddata',
+    'fielddata_frequency_filter',
+    'fields',
+    'format',
+    'ignore_above',
+    'ignore_malformed',
+    'index_options',
+    'index_phrases',
+    'index_prefixes',
+    'index',
+    'meta',
+    'normalizer',
+    'norms',
+    'null_value',
+    'position_increment_gap',
+    'properties',
+    'search_analyzer',
+    'similarity',
+    'store',
+    'term_vector',
+]
+
 CONCAT_TRANSFORM = 'concat'
 MAPPING_TRANSFORM = 'mapping'
 MOVE_TRANSFORM = 'move'
@@ -135,12 +167,21 @@ PLUGIN = 'test_decoding'
 # Trigger function
 TRIGGER_FUNC = 'table_notify'
 
+# Views
+# added underscore to reduce collision
+PRIMARY_KEY_VIEW = '_pkey_view'
+FOREIGN_KEY_VIEW = '_fkey_view'
+# Indices
+PRIMARY_KEY_INDEX = 'pkey_idx'
+FOREIGN_KEY_INDEX = 'fkey_idx'
+
+
 # Primary key delimiter
 PRIMARY_KEY_DELIMITER = '|'
 
 # Replication slot patterns
 LOGICAL_SLOT_PREFIX = re.compile(
-    r'table\s(?P<schema>\w+).\"?(?P<table>\w+)\"?:\s(?P<tg_op>[A-Z]+):'
+    r'table\s\"?(?P<schema>[\w-]+)\"?.\"?(?P<table>[\w-]+)\"?:\s(?P<tg_op>[A-Z]+):'
 )
 LOGICAL_SLOT_SUFFIX = re.compile(
     "\s(?P<key>\"?\w+\"?)\[(?P<type>[\w\s]+)\]:(?P<value>[\w\'\"\-]+)"
