@@ -2,14 +2,22 @@
 import logging
 from collections import defaultdict
 
-from elasticsearch import Elasticsearch
+import boto3
+from elasticsearch import Elasticsearch, RequestsHttpConnection
 from elasticsearch.helpers import parallel_bulk
 from elasticsearch_dsl import Q, Search
 from elasticsearch_dsl.query import Bool
+from requests_aws4auth import AWS4Auth
 
-from .constants import ELASTICSEARCH_TYPES, META
+from .constants import (
+    ELASTICSEARCH_MAPPING_PARAMETERS,
+    ELASTICSEARCH_TYPES,
+    META,
+)
 from .node import traverse_post_order
 from .settings import (
+    ELASTICSEARCH_AWS_HOSTED,
+    ELASTICSEARCH_AWS_REGION,
     ELASTICSEARCH_CA_CERTS,
     ELASTICSEARCH_CHUNK_SIZE,
     ELASTICSEARCH_CLIENT_CERT,
@@ -38,15 +46,9 @@ class ElasticHelper(object):
         host = 'localhost', port = 9200
         """
         url = get_elasticsearch_url()
-        self.__es = Elasticsearch(
-            hosts=[url],
-            timeout=ELASTICSEARCH_TIMEOUT,
-            verify_certs=ELASTICSEARCH_VERIFY_CERTS,
-            use_ssl=ELASTICSEARCH_USE_SSL,
-            ssl_show_warn=ELASTICSEARCH_SSL_SHOW_WARN,
-            ca_certs=ELASTICSEARCH_CA_CERTS,
-            client_cert=ELASTICSEARCH_CLIENT_CERT,
-            client_key=ELASTICSEARCH_CLIENT_KEY,
+        self.__es = get_elasticsearch_client(url)
+        self.version = list(
+            map(int, self.__es.info()['version']['number'].split('.'))
         )
 
     def teardown(self, index):
@@ -93,7 +95,7 @@ class ElasticHelper(object):
         """Refresh the Elasticsearch index."""
         self.__es.indices.refresh(index=indices)
 
-    def _search(self, index, table, fields):
+    def _search(self, index, table, fields=None):
         """
         Search private area for matching docs in Elasticsearch.
 
@@ -104,6 +106,7 @@ class ElasticHelper(object):
             'uid': ['a002', 'a009']
         }
         """
+        fields = fields or {}
         search = Search(using=self.__es, index=index)
         # explicitly exclude all fields since we only need the doc _id
         search = search.source(excludes=['*'])
@@ -127,7 +130,7 @@ class ElasticHelper(object):
         """
         return self.__es.search(index=index, body=body)
 
-    def _create_setting(self, index, node, setting=None):
+    def _create_setting(self, index, node, setting=None, routing=None):
         """Create Elasticsearch setting and mapping if required."""
         body = defaultdict(lambda: defaultdict(dict))
 
@@ -142,7 +145,7 @@ class ElasticHelper(object):
                     }
                 )
 
-            mapping = self._build_mapping(node)
+            mapping = self._build_mapping(node, routing)
             if mapping:
                 body.update(**mapping)
 
@@ -164,7 +167,7 @@ class ElasticHelper(object):
                 f'created setting: {self.__es.indices.get_settings(index)}'
             )
 
-    def _build_mapping(self, root):
+    def _build_mapping(self, root, routing):
         """Get the Elasticsearch mapping from the schema transform."""
 
         for node in traverse_post_order(root):
@@ -179,20 +182,70 @@ class ElasticHelper(object):
                     raise RuntimeError(
                         f'Invalid Elasticsearch type {column_type}'
                     )
-                fields = mapping[column].get('fields')
 
                 if 'properties' not in node._mapping:
                     node._mapping['properties'] = {}
                 node._mapping['properties'][column] = {
                     'type': column_type
                 }
-                if fields:
-                    node._mapping['properties'][column]['fields'] = fields
+
+                for parameter, parameter_value in mapping[column].items():
+                    if parameter == 'type':
+                        continue
+
+                    if parameter not in ELASTICSEARCH_MAPPING_PARAMETERS:
+                        raise RuntimeError(
+                            f'Invalid Elasticsearch mapping parameter '
+                            f'{parameter}'
+                        )
+
+                    node._mapping['properties'][column][
+                        parameter
+                    ] = parameter_value
 
             if node.parent and node._mapping:
                 if 'properties' not in node.parent._mapping:
                     node.parent._mapping['properties'] = {}
                 node.parent._mapping['properties'][node.label] = node._mapping
 
+        if routing:
+            root._mapping['_routing'] = {
+                'required': True
+            }
+
         if root._mapping:
+            if self.version[0] < 7:
+                root._mapping = {'_doc': root._mapping}
+
             return dict(mappings=root._mapping)
+
+
+def get_elasticsearch_client(url):
+
+    if ELASTICSEARCH_AWS_HOSTED:
+        credentials = boto3.Session().get_credentials()
+        http_auth = AWS4Auth(
+            credentials.access_key,
+            credentials.secret_key,
+            ELASTICSEARCH_AWS_REGION,
+            'es',
+            session_token=credentials.token,
+        )
+        return Elasticsearch(
+            hosts=[{'host': url, 'port': 443}],
+            http_auth=http_auth,
+            use_ssl=True,
+            verify_certs=True,
+            connection_class=RequestsHttpConnection,
+        )
+    else:
+        return Elasticsearch(
+            hosts=[url],
+            timeout=ELASTICSEARCH_TIMEOUT,
+            verify_certs=ELASTICSEARCH_VERIFY_CERTS,
+            use_ssl=ELASTICSEARCH_USE_SSL,
+            ssl_show_warn=ELASTICSEARCH_SSL_SHOW_WARN,
+            ca_certs=ELASTICSEARCH_CA_CERTS,
+            client_cert=ELASTICSEARCH_CLIENT_CERT,
+            client_key=ELASTICSEARCH_CLIENT_KEY,
+        )
