@@ -4,7 +4,6 @@ import re
 import sqlalchemy as sa
 from six import string_types
 
-from .base import get_primary_keys
 from .constants import (
     JSONB_OPERATORS,
     NODE_ATTRIBUTES,
@@ -115,10 +114,9 @@ class Node(object):
             self.children = []
 
         self.table_columns = self.model.columns.keys()
-        self._primary_keys = get_primary_keys(self.model)
 
-        if not self._primary_keys:
-            self._primary_keys = kwargs.get('primary_key')
+        if not self.model.primary_keys:
+            setattr(self.model, 'primary_keys', kwargs.get('primary_key'))
 
         # columns to fetch
         self.column_names = [
@@ -131,6 +129,7 @@ class Node(object):
                 str(column) for column in self.table_columns
             ]
             self.column_names.remove('xmin')
+            self.column_names.remove('oid')
 
         if self.label is None:
             self.label = self.table
@@ -185,7 +184,7 @@ class Node(object):
         return [
             getattr(
                 self.model.c, str(sa.text(primary_key))
-            ) for primary_key in self._primary_keys
+            ) for primary_key in self.model.primary_keys
         ]
 
     @property
@@ -300,6 +299,7 @@ class Tree(object):
         return node
 
 
+# TODO: deprecate this method
 def node_from_table(base, table, schema):
     return Node(
         model=base.model(table, schema=schema),
@@ -308,3 +308,21 @@ def node_from_table(base, table, schema):
         label=table,
         primary_key=[],
     )
+
+
+def get_node(tree, table, node_dict):
+
+    root = tree.build(node_dict)
+    for node in traverse_post_order(root):
+        if table == node.table:
+            return node
+        elif table in node.relationship.through_tables:
+            return Node(
+                model=tree.base.model(table, schema=node.schema),
+                table=table,
+                label=table,
+                schema=node.schema,
+                primary_key=[],
+            )
+    else:
+        raise RuntimeError(f'Node for {table} not found')
