@@ -1,7 +1,7 @@
 """PGSync QueryBuilder."""
 import sqlalchemy as sa
 
-from .base import compiled_query, get_foreign_keys, get_primary_keys
+from .base import compiled_query, get_foreign_keys
 from .constants import OBJECT, ONE_TO_MANY, ONE_TO_ONE, SCALAR
 from .exc import FetchColumnForeignKeysError
 from .node import node_from_table
@@ -23,27 +23,26 @@ class QueryBuilder(object):
         self.verbose = verbose
         self.isouter = True
 
-    def _get_foreign_keys(self, model_a, model_b):
-
+    def _get_foreign_keys(self, node_a, node_b):
         if (
-            model_a.relationship.through_tables or
-            model_b.relationship.through_tables
+            node_a.relationship.through_tables or
+            node_b.relationship.through_tables
         ):
 
-            if model_a.through_tables:
-                through_tables = model_a.relationship.through_tables[0]
+            if node_a.relationship.through_tables:
+                through_tables = node_a.relationship.through_tables[0]
 
-            if model_b.through_tables:
-                through_tables = model_b.relationship.through_tables[0]
-                model_a, model_b = model_b, model_a
+            if node_b.relationship.through_tables:
+                through_tables = node_b.relationship.through_tables[0]
+                node_a, node_b = node_b, node_a
 
             through = node_from_table(
                 self.base,
                 through_tables,
-                model_a.schema,
+                node_a.schema,
             )
-            foreign_keys = get_foreign_keys(model_a, through)
-            for key, value in get_foreign_keys(through, model_b).items():
+            foreign_keys = get_foreign_keys(node_a, through)
+            for key, value in get_foreign_keys(through, node_b).items():
                 if key in foreign_keys:
                     foreign_keys[key].extend(value)
                     continue
@@ -53,7 +52,7 @@ class QueryBuilder(object):
 
         else:
 
-            return get_foreign_keys(model_a, model_b)
+            return get_foreign_keys(node_a, node_b)
 
     def _get_column_foreign_keys(
         self,
@@ -197,8 +196,7 @@ class QueryBuilder(object):
                         getattr(
                             child._subquery.c,
                             left_foreign_keys[i],
-                        ) ==
-                        getattr(
+                        ) == getattr(
                             child.parent.model.c,
                             right_foreign_keys[i],
                         )
@@ -232,8 +230,7 @@ class QueryBuilder(object):
                         getattr(
                             child._subquery.c,
                             left_foreign_keys[i],
-                        ) ==
-                        getattr(
+                        ) == getattr(
                             child.parent.model.c,
                             right_foreign_keys[i],
                         )
@@ -416,8 +413,7 @@ class QueryBuilder(object):
                     getattr(
                         child._subquery.c,
                         left_foreign_keys[i],
-                    ) ==
-                    getattr(
+                    ) == getattr(
                         child.parent.model.c,
                         right_foreign_keys[i],
                     )
@@ -488,17 +484,15 @@ class QueryBuilder(object):
         if self.verbose:
             compiled_query(outer_subquery, 'Outer subquery')
 
-        through_primary_keys = get_primary_keys(through.model)
-
         params = []
-        for through_primary_key in through_primary_keys:
+        for primary_key in through.model.primary_keys:
             params.append(
                 sa.func.JSON_BUILD_OBJECT(
-                    str(through_primary_key),
+                    str(primary_key),
                     sa.func.JSON_BUILD_ARRAY(
                         getattr(
                             through.model.c,
-                            through_primary_key,
+                            primary_key,
                         )
                     )
                 )
@@ -557,8 +551,7 @@ class QueryBuilder(object):
                 getattr(
                     outer_subquery.c,
                     left_foreign_keys[i],
-                ) ==
-                getattr(
+                ) == getattr(
                     through.model.c,
                     right_foreign_keys[i],
                 )
@@ -623,8 +616,7 @@ class QueryBuilder(object):
                     getattr(
                         child._subquery.c,
                         foreign_key_columns[i],
-                    ) ==
-                    getattr(
+                    ) == getattr(
                         node.model.c,
                         foreign_keys[node.name][i],
                     )
@@ -668,10 +660,13 @@ class QueryBuilder(object):
                                                 column._orig_key,
                                             ) == column.value
                                         )
+
+            isouter = len(child.parent.children) > 1
+
             from_obj = from_obj.join(
                 child._subquery,
                 onclause=sa.and_(*onclause),
-                isouter=self.isouter,
+                isouter=isouter,
             )
 
         foreign_keys = get_foreign_keys(node.parent, node)
