@@ -18,20 +18,19 @@ import psycopg2
 import sqlalchemy as sa
 
 from . import __version__
-from .base import Base, compiled_query, get_primary_keys
+from .base import Base, compiled_query
 from .constants import (
     DELETE,
     INSERT,
     META,
     PRIMARY_KEY_DELIMITER,
-    SCHEMA,
     TG_OP,
     TRUNCATE,
     UPDATE,
 )
 from .elastichelper import ElasticHelper
-from .exc import RDSError, SuperUserError
-from .node import traverse_breadth_first, traverse_post_order, Tree
+from .exc import RDSError, SchemaError, SuperUserError
+from .node import get_node, traverse_breadth_first, traverse_post_order, Tree
 from .plugin import Plugins
 from .querybuilder import QueryBuilder
 from .redisqueue import RedisQueue
@@ -41,7 +40,7 @@ from .settings import (
     REPLICATION_SLOT_CLEANUP_INTERVAL,
 )
 from .transform import get_private_keys, transform
-from .utils import get_config, progress, show_settings, threaded, Timer
+from .utils import get_config, show_settings, threaded, Timer
 
 logger = logging.getLogger(__name__)
 
@@ -55,15 +54,21 @@ class Sync(Base):
         verbose=False,
         params=None,
         validate=True,
+        repl_slots=True,
     ):
         """Constructor."""
         params = params or {}
         self.index = document['index']
         self.pipeline = document.get('pipeline')
         self.plugins = document.get('plugins', [])
-        self.nodes = document.get('nodes', [])
+        self.nodes = document.get('nodes', {})
         self.setting = document.get('setting')
-        super().__init__(document.get('database', self.index), **params)
+        self.routing = document.get('routing')
+        super().__init__(
+            document.get('database', self.index),
+            verbose=verbose,
+            **params
+        )
         self.es = ElasticHelper()
         self.__name = re.sub(
             '[^0-9a-zA-Z_]+', '', f"{self.database}_{self.index}"
@@ -71,18 +76,24 @@ class Sync(Base):
         self._checkpoint = None
         self._plugins = None
         self._truncate = False
-        self.verbose = verbose
         self._checkpoint_file = f".{self.__name}"
         self.redis = RedisQueue(self.__name)
         self.tree = Tree(self)
         self._last_truncate_timestamp = datetime.now()
         if validate:
-            self.validate()
+            self.validate(repl_slots=repl_slots)
             self.create_setting()
         self.query_builder = QueryBuilder(self, verbose=self.verbose)
 
-    def validate(self):
+    def validate(self, repl_slots=True):
         """Perform all validation right away."""
+
+        # ensure v2 compatible schema
+        if not isinstance(self.nodes, dict):
+            raise SchemaError(
+                'Incompatible schema. Please run v2 schema migration'
+            )
+
         self.connect()
         if self.plugins:
             self._plugins = Plugins('plugins', self.plugins)
@@ -128,30 +139,59 @@ class Sync(Base):
         if self.index is None:
             raise ValueError('Index is missing for document')
 
-        root = self.tree.build(self.nodes[0])
+        # ensure we have bootstrapped and the replication slot exists
+        if repl_slots:
+            if not self.replication_slots(self.__name):
+                raise RuntimeError(
+                    f'Replication slot "{self.__name}" does not exist.\n'
+                    f'Make sure you have run "bootstrap".'
+                )
+
+        root = self.tree.build(self.nodes)
         root.display()
         for node in traverse_breadth_first(root):
             pass
 
     def create_setting(self):
         """Create Elasticsearch setting and mapping if required."""
-        root = self.tree.build(self.nodes[0])
-        self.es._create_setting(self.index, root, setting=self.setting)
+        root = self.tree.build(self.nodes)
+        self.es._create_setting(
+            self.index,
+            root,
+            setting=self.setting,
+            routing=self.routing,
+        )
 
     def setup(self):
         """Create the database triggers and replication slot."""
-        self.teardown()
+        self.teardown(drop_views=False)
 
         for schema in self.schemas:
             tables = set([])
-            root = self.tree.build(self.nodes[0])
+            # tables with user defined foreign keys
+            user_defined_fkey_tables = {}
+
+            root = self.tree.build(self.nodes)
             for node in traverse_breadth_first(root):
                 tables |= set(node.relationship.through_tables)
                 tables |= set([node.table])
-            self.create_triggers(self.database, schema, tables=tables)
+                # we want to get both the parent and the child keys here
+                # even though only one of them is the foreign_key.
+                # this is because we specify both in the schema but
+                # do not indicate which table defines the foreign key.
+                columns = []
+                if node.relationship.foreign_key.parent:
+                    columns.extend(node.relationship.foreign_key.parent)
+                if node.relationship.foreign_key.child:
+                    columns.extend(node.relationship.foreign_key.child)
+                if columns:
+                    user_defined_fkey_tables.setdefault(node.table, set([]))
+                    user_defined_fkey_tables[node.table] |= set(columns)
+            self.create_triggers(schema, tables=tables)
+            self.create_views(schema, tables, user_defined_fkey_tables)
         self.create_replication_slot(self.__name)
 
-    def teardown(self):
+    def teardown(self, drop_views=True):
         """Drop the database triggers and replication slot."""
 
         try:
@@ -161,11 +201,13 @@ class Sync(Base):
 
         for schema in self.schemas:
             tables = set([])
-            root = self.tree.build(self.nodes[0])
+            root = self.tree.build(self.nodes)
             for node in traverse_breadth_first(root):
                 tables |= set(node.relationship.through_tables)
                 tables |= set([node.table])
-            self.drop_triggers(self.database, schema=schema, tables=tables)
+            self.drop_triggers(schema=schema, tables=tables)
+            if drop_views:
+                self.drop_views(schema=schema)
         self.drop_replication_slot(self.__name)
 
     def get_doc_id(self, primary_keys):
@@ -267,7 +309,273 @@ class Sync(Base):
                 payload_data = payload.get('old')
         return payload_data
 
-    def _payloads(self, nodes, index, payloads):
+    def _insert(self, node, root, filters, payloads):
+
+        if node.table in self.tree.nodes:
+
+            if node.table == root.table:
+
+                for payload in payloads:
+                    payload_data = self._payload_data(payload)
+                    primary_values = [
+                        payload_data[key] for key in node.model.primary_keys
+                    ]
+                    primary_fields = dict(
+                        zip(node.model.primary_keys, primary_values)
+                    )
+                    filters[node.table].append({
+                        key: value for key, value in primary_fields.items()
+                    })
+
+            else:
+
+                if not node.parent:
+                    logger.exception(
+                        f'Could not get parent from node: {node.name}'
+                    )
+                    raise
+
+                # set the parent as the new entity that has changed
+                filters[node.parent.table] = []
+                foreign_keys = self.query_builder._get_foreign_keys(
+                    node.parent,
+                    node,
+                )
+
+                for payload in payloads:
+                    payload_data = self._payload_data(payload)
+                    for i, key in enumerate(foreign_keys[node.name]):
+                        value = payload_data[key]
+                        filters[node.parent.table].append({
+                            foreign_keys[node.parent.name][i]: value
+                        })
+
+        else:
+
+            # handle case where we insert into a through table
+            # set the parent as the new entity that has changed
+            filters[node.parent.table] = []
+            foreign_keys = self.query_builder._get_foreign_keys(
+                node.parent,
+                node,
+            )
+
+            for payload in payloads:
+                payload_data = self._payload_data(payload)
+                for i, key in enumerate(foreign_keys[node.name]):
+                    value = payload_data[key]
+                    filters[node.parent.table].append({
+                        foreign_keys[node.parent.name][i]: value
+                    })
+
+        return filters
+
+    def _update(self, node, root, filters, payloads, extra):
+
+        if node.table == root.table:
+
+            # Here, we are performing two operations:
+            # 1) Build a filter to sync the updated record(s)
+            # 2) Delete the old record(s) in Elasticsearch if the
+            #    primary key has changed
+            #   2.1) This is crucial otherwise we can have the old
+            #        and new document in Elasticsearch at the same time
+            docs = []
+            for payload in payloads:
+                payload_data = self._payload_data(payload)
+                primary_values = [
+                    payload_data[key] for key in node.model.primary_keys
+                ]
+                primary_fields = dict(
+                    zip(node.model.primary_keys, primary_values)
+                )
+                filters[node.table].append({
+                    key: value for key, value in primary_fields.items()
+                })
+
+                old_values = []
+                for key in root.model.primary_keys:
+                    if key in payload.get('old').keys():
+                        old_values.append(payload.get('old')[key])
+
+                new_values = [
+                    payload.get('new')[key] for key in root.model.primary_keys
+                ]
+
+                if (
+                    len(old_values) == len(new_values) and
+                    old_values != new_values
+                ):
+                    doc = {
+                        '_id': self.get_doc_id(old_values),
+                        '_index': self.index,
+                        '_op_type': 'delete',
+                    }
+                    if self.routing:
+                        doc['_routing'] = old_values[self.routing]
+                    if self.es.version[0] < 7:
+                        doc['_type'] = '_doc'
+                    docs.append(doc)
+
+            if docs:
+                self.es.bulk(self.index, docs)
+
+        else:
+
+            # update the child tables
+            for payload in payloads:
+                _filters = []
+                fields = collections.defaultdict(list)
+
+                payload_data = self._payload_data(payload)
+
+                primary_values = [
+                    payload_data[key] for key in node.model.primary_keys
+                ]
+                primary_fields = dict(
+                    zip(node.model.primary_keys, primary_values)
+                )
+
+                for key, value in primary_fields.items():
+                    fields[key].append(value)
+                    if None in payload['new'].values():
+                        extra['table'] = node.table
+                        extra['column'] = key
+
+                if None in payload['old'].values():
+                    for key, value in primary_fields.items():
+                        fields[key].append(0)
+
+                for doc_id in self.es._search(self.index, node.table, fields):
+                    where = {}
+                    params = doc_id.split(PRIMARY_KEY_DELIMITER)
+                    for i, key in enumerate(root.model.primary_keys):
+                        where[key] = params[i]
+                    _filters.append(where)
+
+                # also handle foreign_keys
+                if node.parent:
+                    fields = collections.defaultdict(list)
+                    foreign_keys = self.query_builder._get_foreign_keys(
+                        node.parent,
+                        node,
+                    )
+                    foreign_values = [
+                        payload.get('new', {}).get(k) for k in foreign_keys[
+                            node.name
+                        ]
+                    ]
+
+                    for key in [key.name for key in node.primary_keys]:
+                        for value in foreign_values:
+                            if value:
+                                fields[key].append(value)
+                    # TODO: we should combine this with the filter above
+                    # so we only hit Elasticsearch once
+                    for doc_id in self.es._search(
+                        self.index,
+                        node.parent.table,
+                        fields,
+                    ):
+                        where = {}
+                        params = doc_id.split(PRIMARY_KEY_DELIMITER)
+                        for i, key in enumerate(root.model.primary_keys):
+                            where[key] = params[i]
+                        _filters.append(where)
+
+                if _filters:
+                    filters[root.table].extend(_filters)
+
+        return filters
+
+    def _delete(self, node, root, filters, payloads):
+
+        # when deleting a root node, just delete the doc in Elasticsearch
+        if node.table == root.table:
+
+            docs = []
+            for payload in payloads:
+                payload_data = self._payload_data(payload)
+                root_primary_values = [
+                    payload_data[key] for key in root.model.primary_keys
+                ]
+                doc = {
+                    '_id': self.get_doc_id(root_primary_values),
+                    '_index': self.index,
+                    '_op_type': 'delete',
+                }
+                if self.routing:
+                    doc['_routing'] = payload_data[self.routing]
+                if self.es.version[0] < 7:
+                    doc['_type'] = '_doc'
+                docs.append(doc)
+            if docs:
+                self.es.bulk(self.index, docs)
+
+        else:
+
+            # when deleting the child node, find the doc _id where
+            # the child keys match in private, then get the root doc_id and
+            # re-sync the child tables
+            for payload in payloads:
+                payload_data = self._payload_data(payload)
+                primary_values = [
+                    payload_data[key] for key in node.model.primary_keys
+                ]
+                primary_fields = dict(
+                    zip(node.model.primary_keys, primary_values)
+                )
+                fields = collections.defaultdict(list)
+
+                _filters = []
+                for key, value in primary_fields.items():
+                    fields[key].append(value)
+
+                for doc_id in self.es._search(self.index, node.table, fields):
+                    where = {}
+                    params = doc_id.split(PRIMARY_KEY_DELIMITER)
+                    for i, key in enumerate(root.model.primary_keys):
+                        where[key] = params[i]
+
+                    _filters.append(where)
+
+                if _filters:
+                    filters[root.table].extend(_filters)
+
+        return filters
+
+    def _truncate(self, node, root, filters):
+
+        if node.table == root.table:
+
+            docs = []
+            for doc_id in self.es._search(self.index, node.table):
+                doc = {
+                    '_id': doc_id,
+                    '_index': self.index,
+                    '_op_type': 'delete',
+                }
+                if self.es.version[0] < 7:
+                    doc['_type'] = '_doc'
+                docs.append(doc)
+            if docs:
+                self.es.bulk(self.index, docs)
+
+        else:
+
+            _filters = []
+            for doc_id in self.es._search(self.index, node.table):
+                where = {}
+                params = doc_id.split(PRIMARY_KEY_DELIMITER)
+                for i, key in enumerate(root.model.primary_keys):
+                    where[key] = params[i]
+                _filters.append(where)
+            if _filters:
+                filters[root.table].extend(_filters)
+
+        return filters
+
+    def _payloads(self, payloads):
         """
         The "payloads" is a list of payload operations to process together.
 
@@ -296,39 +604,11 @@ class Sync(Base):
         """
         payload = payloads[0]
         tg_op = payload['tg_op']
+        table = payload['table']
         if tg_op not in TG_OP:
             logger.exception(f'Unknown tg_op {tg_op}')
             raise
 
-        table = payload['table']
-        schema = payload['schema']
-
-        model = self.model(table, schema)
-        primary_keys = get_primary_keys(model)
-
-        root_table = nodes[0]['table']
-        root_model = self.model(
-            root_table,
-            nodes[0].get('schema', SCHEMA),
-        )
-        root_primary_keys = get_primary_keys(root_model)
-
-        for payload in payloads:
-            payload_data = self._payload_data(payload)
-            # this is only required for the non truncate tg_ops
-            if payload_data:
-                if not set(primary_keys).issubset(set(payload_data.keys())):
-                    table = payload['table']
-                    schema = payload['schema']
-                    logger.exception(
-                        f'Primary keys {primary_keys} not subset of payload '
-                        f'data {payload_data.keys()} for table '
-                        f'{schema}.{table}'
-                    )
-                    raise
-
-        logger.debug(f'tg_op: {tg_op} table: {schema}.{table}')
-
         # we might receive an event triggered for a table
         # that is not in the tree node.
         # e.g a through table which we need to react to.
@@ -340,261 +620,69 @@ class Sync(Base):
         ):
             return
 
-        filters = {table: [], root_table: []}
+        node = get_node(self.tree, table, self.nodes)
+        root = get_node(self.tree, self.nodes['table'], self.nodes)
+
+        for payload in payloads:
+            payload_data = self._payload_data(payload)
+            # this is only required for the non truncate tg_ops
+            if payload_data:
+                if not set(node.model.primary_keys).issubset(
+                    set(payload_data.keys())
+                ):
+                    logger.exception(
+                        f"Primary keys {node.model.primary_keys} not subset "
+                        f"of payload data {payload_data.keys()} for table "
+                        f"{payload['schema']}.{payload['table']}"
+                    )
+                    raise
+
+        logger.debug(f'tg_op: {tg_op} table: {node.name}')
+
+        filters = {node.table: [], root.table: []}
         extra = {}
 
         if tg_op == INSERT:
 
-            if table in self.tree.nodes:
-
-                if table == root_table:
-                    for payload in payloads:
-                        payload_data = self._payload_data(payload)
-                        primary_values = [
-                            payload_data[key] for key in primary_keys
-                        ]
-                        primary_fields = dict(
-                            zip(primary_keys, primary_values)
-                        )
-                        filters[table].append({
-                            key: value for key, value in primary_fields.items()
-                        })
-                else:
-                    root = self.tree.build(nodes[0])
-                    for node in traverse_post_order(root):
-                        if table != node.table:
-                            continue
-
-                        if not node.parent:
-                            logger.exception(
-                                f'Could not get parent from node: {node.name}'
-                            )
-                            raise
-                        # set the parent as the new entity that has changed
-                        filters[node.parent.table] = []
-                        foreign_keys = self.query_builder._get_foreign_keys(
-                            node.parent,
-                            node,
-                        )
-
-                        _table = self._get_table(schema, table)
-                        node_parent_table = self._get_table(
-                            schema,
-                            node.parent.table,
-                        )
-
-                        for payload in payloads:
-                            payload_data = self._payload_data(payload)
-                            for i, key in enumerate(foreign_keys[_table]):
-                                value = payload_data[key]
-                                filters[node.parent.table].append({
-                                    foreign_keys[node_parent_table][i]: value
-                                })
-
-            else:
-
-                # handle case where we insert into a through table
-                root = self.tree.build(nodes[0])
-                for node in traverse_post_order(root):
-                    if table in node.relationship.through_tables:
-                        if not node.parent:
-                            logger.exception(
-                                f'Could not get parent from node: {node.name}'
-                            )
-                            raise
-                        # set the parent as the new entity that has changed
-                        filters[node.parent.table] = []
-                        foreign_keys = self.query_builder._get_foreign_keys(
-                            node.parent,
-                            node,
-                        )
-
-                        _table = self._get_table(schema, table)
-                        node_parent_table = self._get_table(
-                            schema,
-                            node.parent.table,
-                        )
-
-                        for payload in payloads:
-                            payload_data = self._payload_data(payload)
-                            for i, key in enumerate(foreign_keys[_table]):
-                                value = payload_data[key]
-                                filters[node.parent.table].append({
-                                    foreign_keys[node_parent_table][i]: value
-                                })
-                        break
+            filters = self._insert(
+                node,
+                root,
+                filters,
+                payloads,
+            )
 
         if tg_op == UPDATE:
 
-            if table == root_table:
-                # Here, we are performing two operations:
-                # 1) Build a filter to sync the updated record(s)
-                # 2) Delete the old record(s) in Elasticsearch if the
-                #    primary key has changed
-                #   2.1) This is crucial otherwise we can have the old
-                #        and new document in Elasticsearch at the same time
-                docs = []
-                for payload in payloads:
-                    payload_data = self._payload_data(payload)
-                    primary_values = [
-                        payload_data[key] for key in primary_keys
-                    ]
-                    primary_fields = dict(
-                        zip(primary_keys, primary_values)
-                    )
-                    filters[table].append({
-                        key: value for key, value in primary_fields.items()
-                    })
-
-                    old_values = []
-                    for key in root_primary_keys:
-                        if key in payload.get('old').keys():
-                            old_values.append(payload.get('old')[key])
-
-                    new_values = [
-                        payload.get('new')[key] for key in root_primary_keys
-                    ]
-
-                    if (
-                        len(old_values) == len(new_values) and
-                        old_values != new_values
-                    ):
-                        docs.append({
-                            '_id': self.get_doc_id(old_values),
-                            '_index': index,
-                            '_op_type': 'delete',
-                        })
-
-                if docs:
-                    self.es.bulk(index, docs)
-
-            else:
-
-                # update the child tables
-                for payload in payloads:
-                    payload_data = self._payload_data(payload)
-                    primary_values = [
-                        payload_data[key] for key in primary_keys
-                    ]
-                    primary_fields = dict(
-                        zip(primary_keys, primary_values)
-                    )
-                    fields = collections.defaultdict(list)
-                    _filters = []
-
-                    for key, value in primary_fields.items():
-                        fields[key].append(value)
-                        if None in payload['new'].values():
-                            extra['table'] = table
-                            extra['column'] = key
-
-                    if None in payload['old'].values():
-                        for key, value in primary_fields.items():
-                            fields[key].append(0)
-
-                    for doc_id in self.es._search(index, table, fields):
-                        where = {}
-                        params = doc_id.split(PRIMARY_KEY_DELIMITER)
-                        for i, key in enumerate(root_primary_keys):
-                            where[key] = params[i]
-
-                        _filters.append(where)
-
-                    if _filters:
-                        filters[root_table].extend(_filters)
+            filters = self._update(
+                node,
+                root,
+                filters,
+                payloads,
+                extra,
+            )
 
         if tg_op == DELETE:
 
-            # when deleting a root node, just delete the doc in Elasticsearch
-            if table == root_table:
-
-                docs = []
-                for payload in payloads:
-
-                    payload_data = self._payload_data(payload)
-                    root_primary_values = [
-                        payload_data[key] for key in root_primary_keys
-                    ]
-
-                    docs.append({
-                        '_id': self.get_doc_id(root_primary_values),
-                        '_index': index,
-                        '_op_type': 'delete',
-                    })
-
-                if docs:
-                    self.es.bulk(index, docs)
-
-            else:
-
-                # when deleting the child node, find the doc _id where
-                # the child keys match in private, then get the root doc_id and
-                # re-sync the child tables
-                for payload in payloads:
-                    payload_data = self._payload_data(payload)
-                    primary_values = [
-                        payload_data[key] for key in primary_keys
-                    ]
-                    primary_fields = dict(
-                        zip(primary_keys, primary_values)
-                    )
-                    fields = collections.defaultdict(list)
-                    _filters = []
-                    for key, value in primary_fields.items():
-                        fields[key].append(value)
-
-                    for doc_id in self.es._search(index, table, fields):
-                        where = {}
-                        params = doc_id.split(PRIMARY_KEY_DELIMITER)
-                        for i, key in enumerate(root_primary_keys):
-                            where[key] = params[i]
-
-                        _filters.append(where)
-
-                    if _filters:
-                        filters[root_table].extend(_filters)
+            filters = self._delete(
+                node,
+                root,
+                filters,
+                payloads,
+            )
 
         if tg_op == TRUNCATE:
 
-            if table == root_table:
-                docs = []
-                for doc_id in self.es._search(index, table, {}):
-                    docs.append({
-                        '_id': doc_id,
-                        '_index': index,
-                        '_op_type': 'delete',
-                    })
-                if docs:
-                    self.es.bulk(index, docs)
+            filters = self._truncate(node, root, filters)
 
-            else:
-
-                _filters = []
-                for doc_id in self.es._search(index, table, {}):
-                    where = {}
-                    params = doc_id.split(PRIMARY_KEY_DELIMITER)
-                    for i, key in enumerate(root_primary_keys):
-                        where[key] = params[i]
-
-                    _filters.append(where)
-
-                if _filters:
-                    filters[root_table].extend(_filters)
-
-        # NB: if no filters, then do not execute the sync query.
-        # This is crucial otherwise we would end up performing a full query
+        # If there are no filters, then don't execute the sync query
+        # otherwise we would end up performing a full query
         # and sync the entire db!
         if not any(filters.values()):
             logger.warning('No filters supplied')
             yield {}
             return
 
-        yield self._sync(
-            nodes,
-            index,
-            filters=filters,
-            extra=extra,
-        )
+        yield self._sync(filters=filters, extra=extra)
 
     def _build_filters(self, filters, node):
         """
@@ -627,8 +715,6 @@ class Sync(Base):
 
     def _sync(
         self,
-        nodes,
-        index,
         filters=None,
         txmin=None,
         txmax=None,
@@ -637,7 +723,7 @@ class Sync(Base):
         if filters is None:
             filters = {}
 
-        root = self.tree.build(nodes[0])
+        root = self.tree.build(self.nodes)
 
         self.query_builder.isouter = True
 
@@ -674,42 +760,58 @@ class Sync(Base):
         if self.verbose:
             compiled_query(node._subquery, 'Query')
 
-        row_count = self.query_count(node._subquery)
+        row_count = self.count(node._subquery)
 
-        for i, (keys, row, primary_keys) in enumerate(
-            self.query_yield(node._subquery)
-        ):
+        with click.progressbar(
+            length=row_count,
+            show_pos=True,
+            show_percent=True,
+            show_eta=True,
+            fill_char='=',
+            empty_char='-',
+            width=50,
+        ) as bar:
 
-            progress(i + 1, row_count)
+            for i, (keys, row, primary_keys) in enumerate(
+                self.fetchmany(node._subquery)
+            ):
 
-            row = transform(row, nodes[0])
-            row[META] = get_private_keys(keys)
-            if extra:
-                if extra['table'] not in row[META]:
-                    row[META][extra['table']] = {}
-                if extra['column'] not in row[META][extra['table']]:
-                    row[META][extra['table']][extra['column']] = []
-                row[META][extra['table']][extra['column']].append(0)
+                bar.update(1)
 
-            if self.verbose:
-                print(f'{(i+1)})')
-                print(f'Pkeys: {primary_keys}')
-                pprint.pprint(row)
-                print('-' * 10)
+                row = transform(row, self.nodes)
+                row[META] = get_private_keys(keys)
+                if extra:
+                    if extra['table'] not in row[META]:
+                        row[META][extra['table']] = {}
+                    if extra['column'] not in row[META][extra['table']]:
+                        row[META][extra['table']][extra['column']] = []
+                    row[META][extra['table']][extra['column']].append(0)
 
-            doc = {
-                '_id': self.get_doc_id(primary_keys),
-                '_index': index,
-                '_source': row,
-            }
+                if self.verbose:
+                    print(f'{(i+1)})')
+                    print(f'Pkeys: {primary_keys}')
+                    pprint.pprint(row)
+                    print('-' * 10)
 
-            if self._plugins:
-                doc = list(self._plugins.transform([doc]))[0]
+                doc = {
+                    '_id': self.get_doc_id(primary_keys),
+                    '_index': self.index,
+                    '_source': row,
+                }
 
-            if self.pipeline:
-                doc['pipeline'] = self.pipeline
+                if self.routing:
+                    doc['_routing'] = row[self.routing]
 
-            yield doc
+                if self.es.version[0] < 7:
+                    doc['_type'] = '_doc'
+
+                if self._plugins:
+                    doc = next(self._plugins.transform([doc]))
+
+                if self.pipeline:
+                    doc['pipeline'] = self.pipeline
+
+                yield doc
 
     def sync(self, txmin=None, txmax=None):
         """
@@ -717,15 +819,9 @@ class Sync(Base):
 
         main entry point.
         sync all tables as docs to Elasticsearch
-        document contains -> nodes:
-        nodes contains -> node
+        document contains -> node:
         """
-        docs = self._sync(
-            self.nodes,
-            self.index,
-            txmin=txmin,
-            txmax=txmax,
-        )
+        docs = self._sync(txmin=txmin, txmax=txmax)
         try:
             self.es.bulk(self.index, docs)
         except Exception as e:
@@ -736,17 +832,10 @@ class Sync(Base):
     def sync_payloads(self, payloads):
         """Sync payload when an event is emitted."""
         docs = []
-        for doc in self._payloads(
-            self.nodes,
-            self.index,
-            payloads,
-        ):
+        for doc in self._payloads(payloads):
             docs.append(doc)
         try:
-            self.es.bulk(
-                self.index,
-                itertools.chain(*docs),
-            )
+            self.es.bulk(self.index, itertools.chain(*docs))
         except Exception as e:
             logger.exception(f'Exception: {e}')
             raise
@@ -805,7 +894,7 @@ class Sync(Base):
             ) == ([], [], []):
                 if i % 10 == 0:
                     sys.stdout.write(
-                        f'Polling db {channel}: {j:,} items\n'
+                        f'Polling db {channel}: {j:,} item(s)\n'
                     )
                     sys.stdout.flush()
                 i += 1
@@ -884,7 +973,7 @@ class Sync(Base):
         logger.debug(f'pull txmin: {txmin} txmax: {txmax}')
         # forward pass sync
         self.sync(txmin=txmin, txmax=txmax)
-        # now sync up to txmax to capture everything we might have missed
+        # now sync up to txmax to capture everything we may have missed
         self.logical_slot_changes(txmin=txmin, txmax=txmax)
         self._truncate = True
 
