@@ -289,7 +289,7 @@ class TestNestedChildren(object):
 
     @pytest.fixture(scope='function')
     def nodes(self):
-        return [{
+        return {
             "table": "book",
             "columns": [
                 "isbn",
@@ -410,11 +410,12 @@ class TestNestedChildren(object):
                     }
                 }
             ]
-        }]
+        }
 
     def test_sync(self, sync, nodes, data):
         """test regular sync produces the correct result."""
-        docs = [doc for doc in sync._sync(nodes, 'testdb')]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync()]
         assert len(docs) == 3
         docs = sorted(docs, key=lambda k: k['_id'])
         assert docs == [
@@ -622,7 +623,7 @@ class TestNestedChildren(object):
                 },
             },
         ]
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_insert_root(
         self,
@@ -742,11 +743,8 @@ class TestNestedChildren(object):
             session.add_all(book_shelves)
 
         txmin = sync.checkpoint
-        docs = [
-            doc for doc in sync._sync(
-                nodes, 'testdb', txmin=txmin
-            )
-        ]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync(txmin=txmin)]
         assert len(docs) == 2
         docs = sorted(docs, key=lambda k: k['_id'])
         assert docs == [
@@ -847,7 +845,7 @@ class TestNestedChildren(object):
                 }
             }
         ]
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_update_root(self, data, nodes, book_cls):
         document = {
@@ -868,11 +866,8 @@ class TestNestedChildren(object):
             )
 
         txmin = sync.checkpoint
-        docs = [
-            doc for doc in sync._sync(
-                nodes, 'testdb', txmin=txmin
-            )
-        ]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync(txmin=txmin)]
 
         assert len(docs) == 1
         docs = sorted(docs, key=lambda k: k['_id'])
@@ -940,7 +935,7 @@ class TestNestedChildren(object):
                 }
             }
         ]
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_delete_root(
         self,
@@ -1010,11 +1005,8 @@ class TestNestedChildren(object):
                         sync.es.refresh('testdb')
 
         txmin = sync.checkpoint
-        docs = [
-            doc for doc in sync._sync(
-                nodes, 'testdb', txmin=txmin
-            )
-        ]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync(txmin=txmin)]
         assert len(docs) == 0
 
         docs = search(sync.es, 'testdb')
@@ -1136,7 +1128,7 @@ class TestNestedChildren(object):
                 'title': 'The Rabbit Club'
             }
         ]
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_insert_through_child_noop(self, sync, data):
         # insert a new through child with noop
@@ -1392,7 +1384,7 @@ class TestNestedChildren(object):
                 'title': 'The Rabbit Club'
             }
         ]
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_update_through_child_op(
         self,
@@ -1625,7 +1617,7 @@ class TestNestedChildren(object):
                 'title': 'The Rabbit Club'
             }
         ]
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_delete_through_child_op(self, sync, data, nodes, book_author_cls):
         # delete a new through child with op
@@ -1792,7 +1784,7 @@ class TestNestedChildren(object):
                 'title': 'The Rabbit Club'
             }
         ]
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_insert_nonthrough_child_noop(
         self,
@@ -1835,14 +1827,11 @@ class TestNestedChildren(object):
             session.add(city)
 
         txmin = sync.checkpoint
-        docs = [
-            doc for doc in sync._sync(
-                nodes, 'testdb', txmin=txmin
-            )
-        ]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync(txmin=txmin)]
         assert len(docs) == 0
 
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_update_nonthrough_child_noop(
         self,
@@ -1874,14 +1863,11 @@ class TestNestedChildren(object):
             )
 
         txmin = sync.checkpoint
-        docs = [
-            doc for doc in sync._sync(
-                nodes, 'testdb', txmin=txmin
-            )
-        ]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync(txmin=txmin)]
         assert len(docs) == 0
 
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_delete_nonthrough_child_noop(
         self,
@@ -1913,14 +1899,11 @@ class TestNestedChildren(object):
             )
 
         txmin = sync.checkpoint
-        docs = [
-            doc for doc in sync._sync(
-                nodes, 'testdb', txmin=txmin
-            )
-        ]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync(txmin=txmin)]
         assert len(docs) == 0
 
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_insert_nonthrough_child_op(self, sync, data):
         # insert a new non-through child with op
@@ -1934,9 +1917,8 @@ class TestNestedChildren(object):
         # delete a new non-through child with op
         pass
 
-
     @pytest.fixture(scope='function')
-    def nodes2(self):
+    def node2(self):
         return [{
             "table": "book",
             "columns": [
@@ -2111,14 +2093,11 @@ class TestNestedChildren(object):
                         sync.es.refresh('testdb')
 
         txmin = sync.checkpoint
-        docs = [
-            doc for doc in sync._sync(
-                nodes, 'testdb', txmin=txmin
-            )
-        ]
+        sync.nodes = nodes
+        docs = [doc for doc in sync._sync(txmin=txmin)]
         assert docs == []
         docs = search(sync.es, 'testdb')
-        assert_resync_empty(sync, nodes, 'testdb')
+        assert_resync_empty(sync, nodes)
 
     def test_insert_deep_nested_fk_nonthrough_child_op(
         self,
@@ -2129,7 +2108,7 @@ class TestNestedChildren(object):
         continent_cls,
     ):
         """insert a new deep nested non-through fk child with op."""
-        nodes[0]['children'].append(
+        nodes['children'].append(
             {
                 "table": "book_rating",
                 "columns": [
