@@ -18,19 +18,18 @@ DECLARE
   new_row JSON;
   notification JSON;
   xmin BIGINT;
+
   primary_keys TEXT [] := (
-      SELECT ARRAY_AGG(attname)
-      FROM pg_index
-      JOIN pg_attribute ON attrelid = indrelid AND attnum = ANY(indkey)
-      WHERE indrelid = TG_RELID AND indisprimary
+      SELECT primary_keys
+      FROM _pkey_view
+      WHERE table_name = TG_TABLE_NAME
   );
   foreign_keys TEXT [] := (
-      SELECT ARRAY_AGG(column_name::TEXT)
-      FROM information_schema.key_column_usage
-      WHERE constraint_catalog=current_catalog
-      AND table_name = TG_TABLE_NAME
-      AND position_in_unique_constraint NOTNULL
+      SELECT foreign_keys
+      FROM _fkey_view
+      WHERE table_name = TG_TABLE_NAME
   );
+
 BEGIN
     -- database is also the channel name.
     channel := CURRENT_DATABASE();
@@ -49,14 +48,14 @@ BEGIN
             new_row := (
                 SELECT JSONB_OBJECT_AGG(key, value)
                 FROM JSON_EACH(new_row)
-                WHERE key = ANY(primary_keys) OR key = ANY(foreign_keys)
+                WHERE key = ANY(primary_keys || foreign_keys)
             );
             IF TG_OP = 'UPDATE' THEN
                 old_row = ROW_TO_JSON(OLD);
                 old_row := (
                     SELECT JSONB_OBJECT_AGG(key, value)
                     FROM JSON_EACH(old_row)
-                    WHERE key = ANY(primary_keys) OR key = ANY(foreign_keys)
+                    WHERE key = ANY(primary_keys || foreign_keys)
                 );
             END IF;
             xmin := NEW.xmin;
@@ -103,7 +102,7 @@ $$ LANGUAGE plpgsql;
                 f"JOIN pg_attribute ON attrelid = indrelid AND attnum = ANY(indkey) "
                 f"WHERE indrelid = '{table_name}'::regclass AND indisprimary"
             )
-            rows = pg_base.query(query)[0]
+            rows = pg_base.fetchall(query)[0]
             assert list(rows)[0] == primary_keys
 
     def test_trigger_foreign_key_function(self, connection):
@@ -125,5 +124,5 @@ $$ LANGUAGE plpgsql;
                 f"WHERE constraint_catalog=current_catalog AND "
                 f"table_name='{table_name}' AND position_in_unique_constraint NOTNULL "
             )
-            rows = pg_base.query(query)[0]
+            rows = pg_base.fetchall(query)[0]
             assert rows[0] == foreign_keys
