@@ -8,7 +8,6 @@ from pgsync.utils import (
     get_elasticsearch_url,
     get_postgres_url,
     get_redis_url,
-    progress,
 )
 
 
@@ -16,11 +15,6 @@ from pgsync.utils import (
 class TestUtils(object):
     """Utils tests."""
 
-    @patch('pgsync.utils.sys')
-    def test_progress(self, mock_sys):
-        progress(2, 1, prefix='A', suffix='B', decimals=3, bar_length=5)
-        mock_sys.stdout.write.assert_called_once()
-
     @patch('pgsync.utils.logger')
     def test_get_elasticsearch_url(self, mock_logger):
         assert get_elasticsearch_url() == 'http://localhost:9200'
